from sqlalchemy import create_engine

password = ""
username = ""
db_url = 'postgresql+pg8000://{}:{}@yeraydbinstances.crzn8kwn8uzj.us-east-1.rds.amazonaws.com/openweatherdata'.format(username, password)

engine = create_engine()
engine.connect()




from abc import ABCMeta
from functools import wraps
import time


class GetInfo(metaclass=ABCMeta):
    """
    Abstract class to retrieve information from a Json API
    """
    pass

    def filter_information(self, info_retrieved, schema, result=None):
        """
        :param info_retrieved: Info retrieved from the API
        :param schema: Schema defined by the user to extract partial information
        :param result: Structure containing the filtered information from the API
        :return:
        """
        if result is None:
            result = dict()

        for key in schema:
            if isinstance(info_retrieved, dict):
                value = info_retrieved.get(key, {})
            else:
                for element, sub_info in zip(info_retrieved, schema*len(info_retrieved)):
                    result.append(type(element)())
                    self.filter_information(element, sub_info, result[-1])
                continue

            info_type = type(schema.get(key)) if isinstance(schema.get(key), (list, dict)) else schema.get(key)
            if info_type is not type(value):
                msg = "Types from information schema and the information retrieved does not match {} {}"
                raise ValueError(msg.format(type(schema.get(key)), type(value)))

            if isinstance(value, (list, dict)):
                result[key] = type(value)()
                self.filter_information(info_retrieved.get(key), schema.get(key), result.get(key))
            else:
                if isinstance(result.get(key, None), list):
                    result[key].append(value)
                else:
                    result[key] = value
        return result

    @staticmethod
    def auto_tries(exception_to_catch, tries=4, delay=3):
        """
        Decorator to auto_tries the weather info

        :param tries:
        :param delay: Expressed in seconds, the waiting time between tries
        :return:
        """

        def deco_retry(f):
            @wraps(f)
            def f_retry(*args, **kwargs):
                for internal_tries in range(tries):
                    try:
                        return f(*args, **kwargs)
                    except exception_to_catch:
                        time.sleep(delay)
                return None

            return f_retry

        return deco_retry


import requests
from functools import reduce
from requests.exceptions import RequestException

from open_weather_forecast.get_info import GetInfo


class GetTemperature(GetInfo):

    @GetInfo.auto_tries(RequestException, tries=4, delay=1)
    def http_retrieve(self, url):
        r = requests.get(url)
        if r.ok:
            return r.json()
        else:
            return {}

    def get_temperature(self, url, information_schema):
        temperatures_history = self.http_retrieve(url)

        if not temperatures_history:
            return False, {}
        else:
            try:
                return True, self.filter_information(temperatures_history, information_schema)
            except (ValueError, KeyError):
                return False, {}


def get_from_dict(data_dict, list_of_keys):
    list_of_keys = [list_of_keys] if not isinstance(list_of_keys, list) else list_of_keys
    return reduce(lambda d, k: d.get(k, {}), list_of_keys, data_dict)


def set_a_dict(data_dict, list_of_keys, value):
    get_from_dict(data_dict, list_of_keys[:-1])[list_of_keys[-1]] = value


import unittest
import json
from os import path
from open_weather_forecast.constants import WEATHER_INFORMATION_SCHEMA
from open_weather_forecast.get_info import GetInfo


class FilterInformationTest(unittest.TestCase):
    def setUp(self):
        current_directory = path.abspath(path.dirname(__file__))
        with open(path.join(current_directory, "fixtures/temperature_data.json")) as f:
            self.data = json.load(f)

    def tearDown(self):
        pass

    def basic_test(self):
        # pass
        results = GetInfo().filter_information(self.data, WEATHER_INFORMATION_SCHEMA)
        assert results == {'list': [{'dt_txt': '2015-05-22 15:00:00', 'main': {'temp_max': 293.74, 'temp': 293.74, 'temp_min': 293.437}}, {'dt_txt': '2015-05-22 18:00:00', 'main': {'temp_max': 293.99, 'temp': 293.99, 'temp_min': 293.744}}, {'dt_txt': '2015-05-22 21:00:00', 'main': {'temp_max': 290.95, 'temp': 290.95, 'temp_min': 290.77}}, {'dt_txt': '2015-05-23 00:00:00', 'main': {'temp_max': 288.94, 'temp': 288.94, 'temp_min': 288.817}}, {'dt_txt': '2015-05-23 03:00:00', 'main': {'temp_max': 287.95, 'temp': 287.95, 'temp_min': 287.885}}, {'dt_txt': '2015-05-23 06:00:00', 'main': {'temp_max': 287.805, 'temp': 287.805, 'temp_min': 287.805}}, {'dt_txt': '2015-05-23 09:00:00', 'main': {'temp_max': 291.638, 'temp': 291.638, 'temp_min': 291.638}}, {'dt_txt': '2015-05-23 12:00:00', 'main': {'temp_max': 292.527, 'temp': 292.527, 'temp_min': 292.527}}, {'dt_txt': '2015-05-23 15:00:00', 'main': {'temp_max': 291.49, 'temp': 291.49, 'temp_min': 291.49}}, {'dt_txt': '2015-05-23 18:00:00', 'main': {'temp_max': 290.318, 'temp': 290.318, 'temp_min': 290.318}}, {'dt_txt': '2015-05-23 21:00:00', 'main': {'temp_max': 288.242, 'temp': 288.242, 'temp_min': 288.242}}, {'dt_txt': '2015-05-24 00:00:00', 'main': {'temp_max': 285.211, 'temp': 285.211, 'temp_min': 285.211}}, {'dt_txt': '2015-05-24 03:00:00', 'main': {'temp_max': 282.244, 'temp': 282.244, 'temp_min': 282.244}}, {'dt_txt': '2015-05-24 06:00:00', 'main': {'temp_max': 284.407, 'temp': 284.407, 'temp_min': 284.407}}, {'dt_txt': '2015-05-24 09:00:00', 'main': {'temp_max': 291.455, 'temp': 291.455, 'temp_min': 291.455}}, {'dt_txt': '2015-05-24 12:00:00', 'main': {'temp_max': 294.094, 'temp': 294.094, 'temp_min': 294.094}}, {'dt_txt': '2015-05-24 15:00:00', 'main': {'temp_max': 295.21, 'temp': 295.21, 'temp_min': 295.21}}, {'dt_txt': '2015-05-24 18:00:00', 'main': {'temp_max': 292.971, 'temp': 292.971, 'temp_min': 292.971}}, {'dt_txt': '2015-05-24 21:00:00', 'main': {'temp_max': 290.712, 'temp': 290.712, 'temp_min': 290.712}}]}


class AutoTriesDecoratorTest(unittest.TestCase):
    def setUp(self):
        self.url = 'http://api.openweathermap.org/data/2.5/forecast/city?q={}'.format("London,uk")

    def tearDown(self):
        pass

    def zero_retries_test(self):
        @GetInfo.auto_tries(ValueError, tries=0, delay=0)
        def test_func():
            raise ValueError()

        self.assertEqual(test_func(), None)

    def zero_retries_no_exception_test(self):
        @GetInfo.auto_tries(ValueError, tries=0, delay=0)
        def test_func():
            return 1

        self.assertEqual(test_func(), None)

    def retries_no_exception_test(self):
        @GetInfo.auto_tries(ValueError, tries=1, delay=0)
        def test_func():
            return 1

        self.assertEqual(test_func(), 1)

    def retries_exception_test(self):
        @GetInfo.auto_tries(BaseException, tries=2, delay=0)
        def test_func():
            raise BaseException

        self.assertEqual(test_func(), None)

    def retries_wrong_exception_test(self):
        @GetInfo.auto_tries(ValueError, tries=2, delay=0)
        def test_func():
            raise KeyError

        self.assertRaises(KeyError, test_func)




import unittest
from open_weather_forecast.get_temperature import GetTemperature
from open_weather_forecast.constants import WEATHER_INFORMATION_SCHEMA


class GetTemperatureTest(unittest.TestCase):
    def setUp(self):
        self.url = 'http://api.openweathermap.org/data/2.5/forecast/city?q={}'.format("London,uk")

    def tearDown(self):
        pass

    def basic_test(self):
        pass


from setuptools import setup, find_packages
from codecs import open
from os import path

current_directory = path.abspath(path.dirname(__file__))

# Get the long description from the relevant file
with open(path.join(current_directory, 'DESCRIPTION.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='OpenWeatherForecast',

    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    version='0.1.2',
    description=long_description,
    long_description=long_description,
    url='https://github.com/enanablancaynumeros/weather_forecast',
    author='Yeray Alvarez Romero',
    author_email='yeray.alvarez.romero@gmail.com',
    license='MIT',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Topic :: Office/Business :: Financial',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: MIT License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
    ],

    # What does your project relate to?
    keywords='forecast, weather, prediction',

    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    packages=find_packages(exclude=['contrib', 'docs', 'tests*']),

    # List run-time dependencies here.  These will be installed by pip when
    # your project is installed. For an analysis of "install_requires" vs pip's
    # requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=['requests'],
    tests_require=['nose', 'coveralls'],
    test_suite='nose.collector',
)